// If this code works, it was written by Svetlio Kapralov. If not, I don't know who wrote it.
package net.playerfinder.jsf.components.selectonemenu;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

import javax.faces.component.UIComponent;
import javax.faces.component.ValueHolder;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.model.SelectItem;
import javax.faces.model.SelectItemGroup;
import javax.faces.render.FacesRenderer;

import net.playerfinder.jsf.components.common.Utils;

import com.sun.faces.io.FastStringWriter;
import com.sun.faces.renderkit.Attribute;
import com.sun.faces.renderkit.AttributeManager;
import com.sun.faces.renderkit.RenderKitUtils;
import com.sun.faces.renderkit.html_basic.ListboxRenderer;
import com.sun.faces.util.RequestStateManager;
import com.sun.faces.util.Util;

@FacesRenderer(componentFamily = UISelectOneMenu.COMPONENT_FAMILY, rendererType = UISelectOneMenu.DEFAULT_RENDERER)
public class UISelectOneMenuRenderer extends ListboxRenderer {

	private static final Attribute[] ATTRIBUTES = AttributeManager
			.getAttributes(AttributeManager.Key.SELECTONELISTBOX);
	private static final String ICON_STYLE_PREFIX = "icon_style_";

	@Override
	public void decode(FacesContext context, UIComponent component) {
		super.decode(context, component);
	}

	@Override
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {

		rendererParamsNotNull(context, component);

		if (!shouldEncode(component)) {
			return;
		}

		encodeScript(context, (UISelectOneMenu) component);

		renderSelect(context, component);
	}

	private void encodeScript(FacesContext context,
			UISelectOneMenu selectOneMenu) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		String clientId = selectOneMenu.getClientId(context);
		String styleClassName = ICON_STYLE_PREFIX + clientId.replace(":", "_");

		if (selectOneMenu.getIconsHeight() != null
				|| selectOneMenu.getIconsWidth() != null) {
			int tmpWidth = selectOneMenu.getIconsWidth().intValue() + 5;
			int tmpHeight = selectOneMenu.getIconsHeight().intValue() + 5;

			writer.startElement("style", null);
			writer.write("a." + styleClassName + "{ height: "
					+ Integer.toString(tmpHeight) + "px; } \n");
			writer.write("." + styleClassName
					+ " .ui-selectmenu-item-icon { height: "
					+ selectOneMenu.getIconsHeight() + "px; width: "
					+ Integer.toString(tmpWidth) + "px; } \n");
			writer.write(".ui-selectmenu-menu li." + styleClassName + " a, a."
					+ styleClassName
					+ " span.ui-selectmenu-status { padding-left: "
					+ Integer.toString(tmpWidth) + "px !important; height: "
					+ selectOneMenu.getIconsHeight() + "px; } \n");
			writer.endElement("style");
		}

		String jquerySelector = "pf_jQuery('select[name="
				+ Utils.escapeIdForJQuery(clientId) + "]')";

		writer.startElement("script", null);

		String width = "";
		if (selectOneMenu.getWidth() != null)
			width = "width : " + selectOneMenu.getWidth() + ", \n";
		String menuWidth = "";
		if (selectOneMenu.getMenuWidth() != null)
			menuWidth = "menuWidth : " + selectOneMenu.getMenuWidth() + ", \n";
		String maxHeight = "";
		if (selectOneMenu.getMaxHeight() != null)
			maxHeight = "maxHeight : " + selectOneMenu.getMaxHeight() + ", \n";

		String handleWidth = "handleWidth : "
				+ selectOneMenu.getHandleWidth().toString() + ", \n";
//		String transferClasses = "transferClasses : "
//				+ selectOneMenu.isTransferClasses() + ", \n";
		String transferClasses = "transferClasses : true, \n";

		String icons = "";
		if (selectOneMenu.getIconsHeight() != null
				|| selectOneMenu.getIconsWidth() != null) {
			icons = "icons: [ {find: '." + styleClassName + "'} ],\n";
		}
		
		String formatFunction = "";
		if (selectOneMenu.getFormatFunction() != null) {
			formatFunction = "format: " + selectOneMenu.getFormatFunction() + ",\n";
		}
		
		// selectmenu
		writer.write("pf_jQuery(function() { "
				+ jquerySelector
				+ ".selectmenu({ "
				+ width
				+ menuWidth
				+ maxHeight
				+ handleWidth
				+ transferClasses
				+ icons
				+ formatFunction
				+ "bgImage: function() { return 'url(' + pf_jQuery(this).attr(\"title\") + ')'; }, "
				+ "style : '" + selectOneMenu.getMode() + "' });");

		writer.write("});");
		writer.endElement("script");

	}

	protected boolean renderOption(FacesContext context, UIComponent component,
			Converter converter, SelectItem curItem, Object currentSelections,
			Object[] submittedValues, OptionComponentInfo optionInfo)
			throws IOException {

		Object valuesArray;
		Object itemValue;
		String valueString = getFormattedValue(context, component,
				curItem.getValue(), converter);
		boolean containsValue;
		if (submittedValues != null) {
			containsValue = containsaValue(submittedValues);
			if (containsValue) {
				valuesArray = submittedValues;
				itemValue = valueString;
			} else {
				valuesArray = currentSelections;
				itemValue = curItem.getValue();
			}
		} else {
			valuesArray = currentSelections;
			itemValue = curItem.getValue();
		}

		boolean isSelected = isSelected(context, component, itemValue,
				valuesArray, converter);
		if (optionInfo.isHideNoSelection() && curItem.isNoSelectionOption()
				&& currentSelections != null && !isSelected) {
			return false;
		}

		ResponseWriter writer = context.getResponseWriter();
		assert (writer != null);
		writer.writeText("\t", component, null);
		writer.startElement("option", component);
		writer.writeAttribute("value", valueString, "value");

		if (isSelected) {
			writer.writeAttribute("selected", true, "selected");
		}

		// if the component is disabled, "disabled" attribute would be rendered
		// on "select" tag, so don't render "disabled" on every option.
		if ((!optionInfo.isDisabled()) && curItem.isDisabled()) {
			writer.writeAttribute("disabled", true, "disabled");
		}

		String labelClass;
		if (optionInfo.isDisabled() || curItem.isDisabled()) {
			labelClass = optionInfo.getDisabledClass();
		} else {
			labelClass = optionInfo.getEnabledClass();
		}
		if (labelClass != null) {
			writer.writeAttribute("class", labelClass, "labelClass");
		}

		if (curItem.getDescription() != null) {
			writer.writeAttribute("title", curItem.getDescription(),
					"itemDescription");
			writer.writeAttribute("class", ICON_STYLE_PREFIX
					+ component.getClientId().replace(":", "_"), "class");
		}
		
		if (curItem.isEscape()) {
			String label = curItem.getLabel();
			if (label == null) {
				label = valueString;
			}
			writer.writeText(label, component, "label");
		} else {
			writer.write(curItem.getLabel());
		}

		writer.endElement("option");
		writer.writeText("\n", component, null);
		return true;
	}

	protected int renderOptions(FacesContext context, UIComponent component,
			Iterator<SelectItem> items) throws IOException {

		ResponseWriter writer = context.getResponseWriter();
		assert (writer != null);

		Converter converter = null;
		if (component instanceof ValueHolder) {
			converter = ((ValueHolder) component).getConverter();
		}
		int count = 0;
		Object currentSelections = getCurrentSelectedValues(component);
		Object[] submittedValues = getSubmittedSelectedValues(component);
		Map<String, Object> attributes = component.getAttributes();
		boolean componentDisabled = Util.componentIsDisabled(component);

		OptionComponentInfo optionInfo = new OptionComponentInfo(
				(String) attributes.get("disabledClass"),
				(String) attributes.get("enabledClass"), componentDisabled,
				isHideNoSelection(component));
		RequestStateManager.set(context,
				RequestStateManager.TARGET_COMPONENT_ATTRIBUTE_NAME, component);
		while (items.hasNext()) {
			SelectItem item = items.next();

			if (item instanceof SelectItemGroup) {
				// render OPTGROUP
				writer.startElement("optgroup", component);
				writer.writeAttribute("label", item.getLabel(), "label");

				// if the component is disabled, "disabled" attribute would be
				// rendered
				// on "select" tag, so don't render "disabled" on every option.
				if ((!componentDisabled) && item.isDisabled()) {
					writer.writeAttribute("disabled", true, "disabled");
				}
				count++;
				// render options of this group.
				SelectItem[] itemsArray = ((SelectItemGroup) item)
						.getSelectItems();
				for (int i = 0; i < itemsArray.length; ++i) {
					if (renderOption(context, component, converter,
							itemsArray[i], currentSelections, submittedValues,
							optionInfo)) {
						count++;
					}
				}
				writer.endElement("optgroup");
			} else {
				if (renderOption(context, component, converter, item,
						currentSelections, submittedValues, optionInfo)) {
					count++;
				}
			}
		}

		return count;

	}

	// Render the "select" portion..
	//
	protected void renderSelect(FacesContext context, UIComponent component)
			throws IOException {

		ResponseWriter writer = context.getResponseWriter();
		assert (writer != null);

		if (logger.isLoggable(Level.FINER)) {
			logger.log(Level.FINER, "Rendering 'select'");
		}
		writer.startElement("select", component);
		writeIdAttributeIfNecessary(context, writer, component);
		writer.writeAttribute("name", component.getClientId(context),
				"clientId");
		// render styleClass attribute if present.
		String styleClass;
		if (null != (styleClass = (String) component.getAttributes().get(
				"styleClass"))) {
			writer.writeAttribute("class", styleClass, "styleClass");
		}
		if (!getMultipleText(component).equals("")) {
			writer.writeAttribute("multiple", true, "multiple");
		}

		// Determine how many option(s) we need to render, and update
		// the component's "size" attribute accordingly; The "size"
		// attribute will be rendered as one of the "pass thru" attributes
		Iterator<SelectItem> items = RenderKitUtils.getSelectItems(context,
				component);

		// render the options to a buffer now so that we can determine
		// the size
		FastStringWriter bufferedWriter = new FastStringWriter(128);
		context.setResponseWriter(writer.cloneWithWriter(bufferedWriter));
		int count = renderOptions(context, component, items);
		context.setResponseWriter(writer);
		// If "size" is *not* set explicitly, we have to default it correctly
		Integer size = (Integer) component.getAttributes().get("size");
		if (size == null || size == Integer.MIN_VALUE) {
			size = count;
		}
		writeDefaultSize(writer, size);

		RenderKitUtils.renderPassThruAttributes(context, writer, component,
				ATTRIBUTES, getNonOnChangeBehaviors(component));
		RenderKitUtils.renderXHTMLStyleBooleanAttributes(writer, component);

		RenderKitUtils.renderOnchange(context, component, false);

		// Now, write the buffered option content
		writer.write(bufferedWriter.toString());

		writer.endElement("select");

	}

}
